/* magic.c
xref:
input:
output: stdout
does: - compute nxn magic square for odd n
      - 0-based indexing
      - bechmarking version
*/
/************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
/************************************************************/

/* max n for 4-byte int */
#define N_MAX  1289

#define N N_MAX

typedef int MATRIX [N][N];

MATRIX global_a;

/************************************************************/
int  magic (MATRIX a, int n)
/*
return  -1   if n < 1
return  -2   if n > N_MAX
return  -3   if n is even
return  0    otherwise, and fill a[1:n][1:n] with a magic square
*/
{
  int i, j, k, x = 0, y = n/2;

  /* make sure n is odd, and N_MAX>=n>=1 */
  if (n < 1)     return -1;
  if (n > N_MAX) return -2;
  if (n%2 == 0)  return -3;

  /* initialize matrix to 0 */
  for (i = 0; i < n; ++i)
    for (j = 0; j < n; ++j)
      a[i][j]=0;

  /* magic square algorithm for odd n */
  for (k = 1; k <= n*n; ++k) {
    if      (x <  0 && y <  n)    { x  = n-1;}
    else if (x >= 0 && y >= n)    { y  = 0;}
    else if (x <  0 && y >= n)    { x += 2; --y;}

    if (a[x][y])   { x += 2; --y; } /* square is not empty */

    a[x--][y++]  = k; /* put value k and move up and to the right */
  }

  return 0;
}
/************************************************************/

int main ()
{
  int n, niter = 0;
  clock_t t1, t2;
  double time;
  const double min_time = 10;

  printf("Magic squares, odd n, 1 to %d\n", N);

  t1 = clock();       /* start the clock */
  do {
    for (n=1; n <= N; n += 2)    magic(global_a, n);
    t2 = clock();
    time =  (t2 - t1) / (double) CLOCKS_PER_SEC;
    ++niter;
  } while (time < min_time);

  printf("Time = %f seconds per iteration.\n", time / niter);

  return 0;
}
